################################################################################
################################################################################
###
###  This file is automatically generated. Do not change this file! Changes
###  will get overwritten! Change the source file for "setup.py" instead.
###  This is either 'packageinfo.json' or 'packageinfo.jsonc'
###
################################################################################
################################################################################


from setuptools import setup

def readme():
	with open("README.md", "r", encoding="UTF-8-sig") as f:
		return f.read()

setup(
	author = "Jürgen Knauth",
	author_email = "pubsrc@binary-overflow.de",
	classifiers = [
		"Development Status :: 5 - Production/Stable",
		"License :: OSI Approved :: Apache Software License",
		"Programming Language :: Python :: 3",
		"Topic :: Software Development :: Testing",
	],
	description = "This python module provides a simple to use infrastructure for running unit tests.",
	include_package_data = True,
	install_requires = [
		"jk_logging",
		"jk_temporary",
		"jk_json",
		"graphviz",
		"jinja2",
		"jk_terminal_essentials",
	],
	keywords = [
		"testing",
		"unittests",
		"tests",
	],
	license = "Apache2",
	name = "jk_testing",
	package_data = {
		"": [
			"data/*",
			"data/html_default/*",
			"data/html_default/files/*",
			"data/html_default/files/images/*",
			"data/html_default/templates/*",
		],
	},
	packages = [
		"jk_testing",
		"jk_testing.utils",
	],
	version = "0.2021.5.25",
	zip_safe = False,
	long_description = readme(),
	long_description_content_type="text/markdown",
)
